@@ -15,6 +15,21 @@ use Moose::Role;
 sub format_number {
 	my ($self, $number, $format, $currency, $for_cash) = @_;
 	
+	# Check if the locales numbering system is algorithmic. If so ignore the format
+	my $numbering_system = $self->default_numbering_system();
+	if ($self->numbering_system->{$numbering_system}{type} eq 'algorithmic') {
+		$format = $self->numbering_system->{$numbering_system}{data};
+		return $self->_algorithmic_number_format($number, $format);
+	}
+	
+	$format //= '0';
+	
+	return $self->_format_number($number, $format, $currency, $for_cash);
+}
+
+sub _format_number {
+	my ($self, $number, $format, $currency, $for_cash) = @_;
+	
 	# First check to see if this is an algorithmic format
 	my @valid_formats = $self->_get_valid_algorithmic_formats();
 	
@@ -22,14 +37,6 @@ sub format_number {
 		return $self->_algorithmic_number_format($number, $format);
 	}
 	
-	# Check if the locales numbering system is algorithmic. If so ignore the format
-	my $numbering_system = $self->default_numbering_system();
-	if ($self->numbering_system->{$numbering_system}{type} eq 'algorithmic') {
-		$format = $self->numbering_system->{$numbering_system}{data};
-	}
-	
-	$format //= '0';
-
 	# Some of these algorithmic formats are in locale/type/name format
 	if (my ($locale_id, $type, $format) = $format =~ m(^(.*?)/(.*?)/(.*?)$)) {
 		my $locale = Locale::CLDR->new($locale_id);
@@ -276,7 +283,7 @@ sub get_formatted_number {
 	my ($separator, $decimal_point) = ($symbols{$symbols_type}{group}, $symbols{$symbols_type}{decimal});
 	my ($minor_group, $major_group) = ($format->{$type}{minor_group}, $format->{$type}{major_group});
 	
-	if (defined $minor_group) {
+	if (defined $minor_group && $separator) {
 		# Fast commify using unpack
 		my $pattern = "(A$minor_group)(A$major_group)*";
 		$number = reverse join $separator, grep {length} unpack $pattern, reverse $integer;
@@ -396,7 +403,7 @@ sub _process_algorithmic_number_data {
 			}
 			else {
 				# Assume a format
-				$format_rule =~ s/→(.+)→/$self->format_number($positive_number, $1)/e;
+				$format_rule =~ s/→(.+)→/$self->_format_number($positive_number, $1)/e;
 			}
 		}
 		elsif($format_rule =~ /=%%.*=/) {
@@ -406,13 +413,13 @@ sub _process_algorithmic_number_data {
 			$format_rule =~ s/=%(.*?)=/$self->_algorithmic_number_format($number, $1, 'public')/eg;
 		}
 		elsif($format_rule =~ /=.*=/) {
-			$format_rule =~ s/=(.*?)=/$self->format_number($number, $1)/eg;
+			$format_rule =~ s/=(.*?)=/$self->_format_number($number, $1)/eg;
 		}
 	}
 	# Fractions
 	elsif( $number =~ /\./ ) {
 		my $in_fraction_rule_set = 1;
-		my ($integer, $fraction) = $number =~ /^([^.])*\.(.*)$/;
+		my ($integer, $fraction) = $number =~ /^([^.]*)\.(.*)$/;
 		
 		if ($number >= 0 && $number < 1) {
 			$format_rule =~ s/\[.*\]//;
@@ -434,7 +441,7 @@ sub _process_algorithmic_number_data {
 				$format_rule =~ s/→(.*)→/$self->_process_algorithmic_number_data_fractions($fraction, $format_data)/e;
 			}
 			else {
-				$format_rule =~ s/→(.*)→/$self->format_number($fraction, $1)/e;
+				$format_rule =~ s/→(.*)→/$self->_format_number($fraction, $1)/e;
 			}
 		}
 		
@@ -451,7 +458,7 @@ sub _process_algorithmic_number_data {
 				$format_rule =~ s/←(.*)←/$self->_process_algorithmic_number_data($integer, $format_data, $in_fraction_rule_set)/e;
 			}
 			else {
-				$format_rule =~ s/←(.*)←/$self->format_number($integer, $1)/e;
+				$format_rule =~ s/←(.*)←/$self->_format_number($integer, $1)/e;
 			}
 		}
 		
@@ -463,7 +470,7 @@ sub _process_algorithmic_number_data {
 				$format_rule =~ s/=%(.*?)=/$self->_algorithmic_number_format($number, $1, 'public')/eg;
 			}
 			else {
-				$format_rule =~ s/=(.*?)=/$self->format_number($integer, $1)/eg;
+				$format_rule =~ s/=(.*?)=/$self->_format_number($integer, $1)/eg;
 			}
 		}
 	}
@@ -496,7 +503,7 @@ sub _process_algorithmic_number_data {
 						$format_rule =~ s/←(.*)←/$self->_process_algorithmic_number_data($number * $base_value, $format_data, $in_fraction_rule_set)/e;
 					}
 					else {
-						$format_rule =~ s/←(.*)←/$self->format_number($number * $base_value, $1)/e;
+						$format_rule =~ s/←(.*)←/$self->_format_number($number * $base_value, $1)/e;
 					}
 				}
 				else {
@@ -504,7 +511,7 @@ sub _process_algorithmic_number_data {
 				}
 			}
 			elsif($format_rule =~ /=.*=/) {
-				$format_rule =~ s/=(.*?)=/$self->format_number($number, $1)/eg;
+				$format_rule =~ s/=(.*?)=/$self->_format_number($number, $1)/eg;
 			}
 		}
 		else {
@@ -519,7 +526,7 @@ sub _process_algorithmic_number_data {
 						$format_rule =~ s/→(.+)→/$self->_process_algorithmic_number_data($number % $divisor, $format_data)/e;
 					}
 					else {
-						$format_rule =~ s/→(.*)→/$self->format_number($number % $divisor, $1)/e;
+						$format_rule =~ s/→(.*)→/$self->_format_number($number % $divisor, $1)/e;
 					}
 				}
 				else {
@@ -538,7 +545,7 @@ sub _process_algorithmic_number_data {
 						$format_rule =~ s|←(.*)←|$self->_process_algorithmic_number_data(int ($number / $divisor), $format_data)|e;
 					}
 					else {
-						$format_rule =~ s|←(.*)←|$self->format_number(int($number / $divisor), $1)|e;
+						$format_rule =~ s|←(.*)←|$self->_format_number(int($number / $divisor), $1)|e;
 					}
 				}
 				else {
@@ -554,7 +561,7 @@ sub _process_algorithmic_number_data {
 					$format_rule =~ s/=%(.*?)=/$self->_algorithmic_number_format($number, $1, 'public')/eg;
 				}
 				else {
-					$format_rule =~ s/=(.*?)=/$self->format_number($number, $1)/eg;
+					$format_rule =~ s/=(.*?)=/$self->_format_number($number, $1)/eg;
 				}
 			}
 		}
@@ -597,4 +604,3 @@ no Moose::Role;
 1;
 
 # vim: tabstop=4
-
